﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Effects;
using System.Windows.Shapes;
using System.Windows.Interactivity;
using Microsoft.Expression.Controls;
using Microsoft.Expression.Media;
using System.Windows.Input;

namespace DiagramLibrary.Connector
{
    public class ConnectorBehavior : Behavior<FrameworkElement>
    {
        private List<Connection> _connections = new List<Connection>();
        private LineArrow _connection;
        private bool _isMouseDown = false;
        private bool _drawing = false;
        private Point _start;
        private Canvas _root;

        #region Properties
        public List<Connection> Connections
        {
            get { return _connections; }
        }
        #endregion

        #region ctor

        public ConnectorBehavior()
        {

        }
        #endregion

        #region Overrides

        protected override void OnAttached()
        {
            base.OnAttached();
            this.AssociatedObject.Loaded += new RoutedEventHandler(AssociatedObject_Loaded);
            
            //this.AssociatedObject.MouseLeftButtonDown += OnMouseLeftButtonDown;
            //this.AssociatedObject.MouseLeftButtonUp += OnMouseLeftButtonUp;
            //this.AssociatedObject.MouseMove += OnMouseMove;
        }

        void AssociatedObject_Loaded(object sender, RoutedEventArgs e)
        {
            _root = GetCanvas(this.AssociatedObject);
            if (_root != null)
            {
                _root.MouseLeftButtonDown += OnMouseLeftButtonDown;
                _root.MouseLeftButtonUp += OnMouseLeftButtonUp;
                _root.MouseMove += OnMouseMove;

                
                
                /*
                // transform the mouse move event into an observable source of screen coordinates
                var mouseMoveEvent = Observable.FromEvent<MouseEventArgs>(_root, "MouseMove");
                //var mouseMoveEvent = Observable.FromEvent<MouseEventArgs>(root, "MouseMove")
                //                                .Select(e => e.EventArgs.GetPosition(root));
                // create observable sources from the left button events
                var mouseLeftButtonDown = Observable.FromEvent<MouseButtonEventArgs>(_root, "MouseLeftButtonDown");
                var mouseLeftButtonUp = Observable.FromEvent<MouseButtonEventArgs>(_root, "MouseLeftButtonUp");

                // create a 'drag event', which takes the delta in mouse movements when the left button is down
                //var draggingEvents = mouseMoveEvent.SkipUntil(mouseLeftButtonDown)
                //                                    .TakeUntil(mouseLeftButtonUp)
                //                                    .Let(mm => mm.Zip(mm.Skip(1), (prev, cur) =>
                //                                        new
                //                                        {
                //                                            X2 = cur.X,
                //                                            X1 = prev.X,
                //                                            Y2 = cur.Y,
                //                                            Y1 = prev.Y
                //                                        })).Repeat();


                mouseLeftButtonDown.Subscribe(
                    p =>
                    {
                        _drawing = true;
                        _start = p.EventArgs.GetPosition((UIElement)p.Sender);
                        Point connector = new Point(this.AssociatedObject.Left(_root), this.AssociatedObject.Top(_root));
                        if (_start.X >= connector.X &&
                            _start.Y >= connector.Y &&
                            _start.X <= connector.X + this.AssociatedObject.Width &&
                            _start.Y <= connector.Y + this.AssociatedObject.Height)
                        {

                        LineArrow line = new LineArrow()
                        {
                            Width = 1,
                            Height = 1,
                            BendAmount = 0.0d,
                            Stroke = new SolidColorBrush(Colors.Black)
                        };
                        line.SetValue(Canvas.LeftProperty, _start.X);
                        line.SetValue(Canvas.TopProperty, _start.Y);
                        _root.Children.Add(line);
                        }
                    });

                mouseLeftButtonUp.Subscribe(
                    p =>
                    {
                        _drawing = false;
                    });

            // subscribe and draw lines
                mouseMoveEvent.Subscribe(
                p =>
                {
                    if (_drawing)
                    {
                        Point current = p.EventArgs.GetPosition((UIElement)p.Sender);
                        //Line line = new Line()
                        //{
                        //    X1 = _start.X,
                        //    Y1 = _start.Y,
                        //    X2 = current.X,
                        //    Y2 = current.Y
                        //};
                        //line.Stroke = new SolidColorBrush(Colors.Black);
                        //_root.Children.Add(line);

                        //connectionObject.Line.SetValue(Canvas.LeftProperty, left);
                        //            connectionObject.Line.SetValue(Canvas.TopProperty, top);
                        //            connectionObject.Line.Height = Math.Abs(height);
                        //            connectionObject.Line.Width = Math.Abs(width);
                        //_start = current;
                    }
                });

            */
            }



            //Setup();
        }

        protected override void OnDetaching()
        {
            base.OnDetaching();
            _root.MouseLeftButtonDown -= OnMouseLeftButtonDown;
            _root.MouseLeftButtonUp -= OnMouseLeftButtonUp;
            _root.MouseMove -= OnMouseMove;

            //this.AssociatedObject.MouseLeftButtonDown -= OnMouseLeftButtonDown;
            //this.AssociatedObject.MouseLeftButtonUp -= OnMouseLeftButtonUp;
            //this.AssociatedObject.MouseMove -= OnMouseMove;
        }
    
        #endregion

        private void OnMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (_connection == null)
            {
                _isMouseDown = true;
                _start = e.GetPosition((UIElement)sender);
                Point connector = new Point(this.AssociatedObject.Left(_root), this.AssociatedObject.Top(_root));
                if (_start.X >= connector.X &&
                    _start.Y >= connector.Y &&
                    _start.X <= connector.X + this.AssociatedObject.Width &&
                    _start.Y <= connector.Y + this.AssociatedObject.Height)
                {
                    

                    LineArrow line = new LineArrow()
                    {
                        Width = 1,
                        Height = 1,
                        BendAmount = 0.0d,
                        Stroke = new SolidColorBrush(Colors.Black),
                        StartCorner = CornerType.TopLeft
                    };
                    line.SetValue(Canvas.LeftProperty, _start.X);
                    line.SetValue(Canvas.TopProperty, _start.Y);

                    _connection = line;

                    _root.Children.Add(line);
                }
            }
        }
        private void OnMouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            _isMouseDown = false;
        }
        private void OnMouseMove(object sender, MouseEventArgs e)
        {
            if (_isMouseDown && _connection != null)
            {
                Point current = e.GetPosition((UIElement)sender);
                //_connection.Height = Math.Abs(current.Y - _start.Y);
                //_connection.Width = Math.Abs(current.X - _start.X);
                _connection.Height = (current.Y - _start.Y);
                _connection.Width = (current.X - _start.X);
            }
        }


        void _connection_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            _isMouseDown = false;
        }
        void _connection_MouseMove(object sender, MouseEventArgs e)
        {
            if (_isMouseDown)
            {
                Canvas root = GetCanvas(this.AssociatedObject);
                if (root != null)
                {
                    Point ptMouse = e.GetPosition(root);
                    _connection.Height = Math.Abs(ptMouse.Y - (this.AssociatedObject.Top() + this.AssociatedObject.Height));
                    _connection.Width = Math.Abs(ptMouse.X - (this.AssociatedObject.Left() + this.AssociatedObject.Width));
                }

            }
        }
        void _connection_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            _isMouseDown = true;
        }


        private void Setup()
        {
            Canvas rootCanvas = GetCanvas(this.AssociatedObject);

            // transform the mouse move event into an observable source of screen coordinates
            var mouseMoveEvent = Observable.FromEvent<MouseEventArgs>(rootCanvas, "MouseMove")
                                            .Select(e => e.EventArgs.GetPosition(rootCanvas));

            // create observable sources from the left button events
            var mouseLeftButtonDown = Observable.FromEvent<MouseButtonEventArgs>(rootCanvas, "MouseLeftButtonDown");
            var mouseLeftButtonUp = Observable.FromEvent<MouseButtonEventArgs>(rootCanvas, "MouseLeftButtonUp");

            // create a 'drag event', which takes the delta in mouse movements when the left button is down
            var draggingEvents = mouseMoveEvent.SkipUntil(mouseLeftButtonDown)
                                                .TakeUntil(mouseLeftButtonUp)
                                                .Let(mm => mm.Zip(mm.Skip(1), (prev, cur) =>
                                                    new
                                                    {
                                                        X2 = cur.X,
                                                        X1 = prev.X,
                                                        Y2 = cur.Y,
                                                        Y1 = prev.Y
                                                    })).Repeat();

            mouseLeftButtonDown.Subscribe(
               p =>
               {
                   var connectFromX = this.AssociatedObject.Left();
                   var connectFromY = this.AssociatedObject.Top();
                   var adjY = 10;
                   var adjX = 5;
                   LineArrow line = new LineArrow()
                   {
                       Width = 1,
                       Height = 1,
                       BendAmount = 0.0d,
                       Stroke = new SolidColorBrush(Colors.Black)
                   };
                   line.SetValue(Canvas.LeftProperty, connectFromX + adjX);
                   line.SetValue(Canvas.TopProperty, connectFromY - (line.Height / 2 + adjY));

                   Canvas root = GetCanvas(this.AssociatedObject);
                   if (root != null)
                   {
                       root.Children.Add(line);
                   }

                   Connection connectionObject = new Connection()
                   {
                       FromElement = this.AssociatedObject,
                       ToElement = null,
                       Line = line
                   };
                   Connections.Add(connectionObject);
               });

            // subscribe and draw lines
            //draggingEvents.Subscribe(
            //    p =>
            //    {
            //        var connectFromX = this.AssociatedObject.Left();
            //        var connectFromY = this.AssociatedObject.Top();
            //        var adjY = 10;
            //        var adjX = 5;

            //        Connection connectionObject = Connections.FirstOrDefault();

            //        if (connectionObject != null)
            //        {
            //            var top = connectFromY - (connectionObject.Line.Height / 2 + adjY);
            //            var left = connectFromX + adjX;

            //            double height = Math.Abs(p.Y1 - top);
            //            double width = Math.Abs(p.X1 - left);

            //            connectionObject.Line.SetValue(Canvas.LeftProperty, left);
            //            connectionObject.Line.SetValue(Canvas.TopProperty, top);
            //            connectionObject.Line.Height = Math.Abs(height);
            //            connectionObject.Line.Width = Math.Abs(width);
            //        }
            //    });
        }

        private Canvas GetCanvas(FrameworkElement element)
        {
            DependencyObject obj = VisualTreeHelper.GetParent(element);
            if (obj != null && obj is Canvas)
            {
                return obj as Canvas;
            }
            else
            {
                return GetCanvas(element.Parent as FrameworkElement);
            }
        }

    };
}
