﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections;
using System.Linq;

using Telerik.Windows.Controls.DragDrop;

namespace DiagramLibrary.Diagram
{
    [TemplatePart(Name = "PanningContainer", Type = typeof(Canvas))]
    [TemplatePart(Name = "ItemsContainer", Type = typeof(Canvas))]
    [TemplatePart(Name = "ConnectorsContainer", Type = typeof(Canvas))]
    [TemplatePart(Name = "ConnectionsContainer", Type = typeof(Canvas))]
    public class DiagramPanel : Control
    {

        #region Member Variables

        private DiagramItem _selectedContainer;
        private bool _isPanning;
        private Point _startPosition;

        #endregion

        #region Dependency Properties

        public static readonly DependencyProperty ConnectionStyleProperty = DependencyProperty.Register(
            "ConnectionStyle", typeof(Style), typeof(DiagramPanel), null);
        public Style ConnectionStyle
        {
            get { return (Style)GetValue(ConnectionStyleProperty); }
            set { SetValue(ConnectionStyleProperty, value); }
        }

        public static readonly DependencyProperty ItemsSourceProperty = DependencyProperty.Register(
            "ItemsSource", typeof(IEnumerable), typeof(DiagramPanel), null);
        public IEnumerable ItemsSource
        {
            get { return (IEnumerable)GetValue(ItemsSourceProperty); }
            set { SetValue(ItemsSourceProperty, value); }
        }

        public static readonly DependencyProperty ItemTemplateProperty = DependencyProperty.Register(
            "ItemTemplate", typeof(DataTemplate), typeof(DiagramPanel), null);
        public DataTemplate ItemTemplate
        {
            get { return (DataTemplate)GetValue(ItemTemplateProperty); }
            set { SetValue(ItemTemplateProperty, value); }
        }

        public static readonly DependencyProperty SelectedItemProperty = DependencyProperty.Register(
            "SelectedItem", typeof(object), typeof(DiagramPanel), null);
        public object SelectedItem
        {
            get { return GetValue(SelectedItemProperty); }
            set { SetValue(SelectedItemProperty, value); }
        }

        #endregion

        protected Canvas ConnectionsContainer { get; set; }
		protected Canvas ItemsContainer { get; set; }


        #region ctor

        public DiagramPanel()
		{
            DefaultStyleKey = typeof(DiagramPanel);
		}

        #endregion

        public override void OnApplyTemplate()
        {
            PanningContainer = GetTemplateChild("PanningContainer") as Canvas;
            PanningContainer.MouseLeftButtonDown += OnPanStart;
            PanningContainer.MouseMove += OnPanMove;
            PanningContainer.MouseLeftButtonUp += OnPanEnd;

            ConnectorsContainer = GetTemplateChild("ConnectorsContainer") as Canvas;
            ConnectorsContainer.RenderTransform = new TranslateTransform();

            ConnectionsContainer = GetTemplateChild("ConnectionsContainer") as Canvas;
            ConnectionsContainer.RenderTransform = new TranslateTransform();

            ItemsContainer = GetTemplateChild("ItemsContainer") as Canvas;
            ItemsContainer.RenderTransform = new TranslateTransform();

            RadDragAndDropManager.AddDragInfoHandler(this, OnDragInfo);
            RadDragAndDropManager.AddDragQueryHandler(this, OnDragQuery);
            RadDragAndDropManager.AddDropInfoHandler(this, OnDropInfo);
            RadDragAndDropManager.AddDropQueryHandler(this, OnDropQuery);

            Render();
        }

        #region Drag and Drop

        public void OnDragInfo(object sender, DragDropEventArgs e)
        {
            var source = e.Options.Source as Connector;
        }
        public void OnDragQuery(object sender, DragDropQueryEventArgs e)
        {
            var source = e.Options.Source as Connector;

            if (e.Options.Status == DragStatus.DragQuery && 
                source != null)
            {
                source.IsSelected = true;
                e.Options.Payload = source;
                //ContentControl cue = new ContentControl();
                ////cue.ContentTemplate = this.Resources["ApplicationDragTemplate"] as DataTemplate;
                //cue.Content = source;
                //e.Options.DragCue = cue;
                e.Options.ArrowCue = RadDragAndDropManager.GenerateArrowCue();
            }

            e.QueryResult = true;
        }
        public void OnDropInfo(object sender, DragDropEventArgs e)
        {
            var source = e.Options.Payload as Connector;
            var target = e.Options.Destination as Connector;
            if (source == null) return;

            if (target != null)
            {
                target.IsMouseOver = true;

                if (e.Options.Status == DragStatus.DropComplete)
                {
                    source.IsSelected = source.ParentDiagramItem.IsSelected;
                    var connection = new Connection()
                    {
                        Source = source,
                        Sink = target
                    };
                    connection.Path.Style = ConnectionStyle;

                    target.ParentDiagramItem.Connections.Add(connection);
                    target.ParentDiagramItem.ParentItem = source.ParentDiagramItem;
                    target.ParentDiagramItem.ParentId = source.ParentDiagramItem.Id;
                    source.ParentDiagramItem.ChildItems.Add(target.ParentDiagramItem);
                    ConnectionsContainer.Children.Add(connection.Path2);

                    // Positioning
                    Dispatcher.BeginInvoke(() =>
                    {
                        target.ParentDiagramItem.UpdateConnectionPath(connection);
                    });
                }
            }
            else
            {
                source.IsSelected = false;
            }
        }
        public void OnDropQuery(object sender, DragDropQueryEventArgs e)
        {
            //var source = e.Options.Payload as Connector;
            //var target = e.Options.Destination as Connector;
            if (e.Options.Destination is Connector)
            {
                e.QueryResult = true;
            }
        }

        #endregion

        #region Connectors Container

        protected Canvas ConnectorsContainer { get; set; }

        #endregion

        #region Pan Container

        protected Canvas PanningContainer { get; set; }
        private void OnPanStart(object sender, MouseButtonEventArgs e)
        {
            if ((Keyboard.Modifiers & ModifierKeys.Control) == ModifierKeys.Control)
            {
                _isPanning = true;
                _startPosition = e.GetPosition(this);
                PanningContainer.CaptureMouse();
            }
            else
            {
                ClearSelection();
            }
        }
        private void OnPanMove(object sender, MouseEventArgs e)
        {
            var isControlPressed = (Keyboard.Modifiers & ModifierKeys.Control) == ModifierKeys.Control;
            if (_isPanning && isControlPressed)
            {
                var currentPosition = e.GetPosition(this);
                var delta = new Point(currentPosition.X - _startPosition.X, currentPosition.Y - _startPosition.Y);
                _startPosition = currentPosition;

                var cntrXform = ConnectorsContainer.RenderTransform as TranslateTransform;
                var connXform = ConnectionsContainer.RenderTransform as TranslateTransform;
                var itemXform = ItemsContainer.RenderTransform as TranslateTransform;

                cntrXform.X += delta.X;
                cntrXform.Y += delta.Y;

                connXform.X += delta.X;
                connXform.Y += delta.Y;

                itemXform.X += delta.X;
                itemXform.Y += delta.Y;
            }
        }
        private void OnPanEnd(object sender, MouseButtonEventArgs e)
        {
            _isPanning = false;
            PanningContainer.ReleaseMouseCapture();
        }

        #endregion

        #region Render

        private void Render()
        {
            if (ItemsContainer != null)
            {
                // First pass, create the DiagramItem.
                foreach (var item in ItemsSource)
                {
                    var current = item;
                    IPosition pos = current as IPosition;
                    DiagramItem newItem = new DiagramItem()
                    {
                        Id = pos.Id,
                        Content = current,
                        ContentTemplate = ItemTemplate,
                        Owner = this,
                        ParentId = pos.ParentId,
                        Left = pos.Left,
                        Top = pos.Top
                    };

                    Connector topCenterConnector = new Connector()
                    {
                        AllowDrop = true,
                        Height = 10,
                        Width = 10,
                        Owner = this,
                        ParentDiagramItem = newItem,
                        Orientation = ConnectorOrientation.Top
                    };
                    RadDragAndDropManager.SetAllowDrop(topCenterConnector, true);
                    newItem.Connectors.Add(topCenterConnector);

                    Connector bottomCenterConnector = new Connector()
                    {
                        Height = 10,
                        Width = 10,
                        Owner = this,
                        ParentDiagramItem = newItem,
                        Orientation = ConnectorOrientation.Bottom
                    };
                    RadDragAndDropManager.SetAllowDrag(bottomCenterConnector, true);
                    newItem.Connectors.Add(bottomCenterConnector);

                    //var connection = new Connection()
                    //{
                    //    Source = topCenterConnector
                    //};
                    //connection.Path.Style = ConnectionStyle;
                    //newItem.Connections.Add(connection);

                    //AddItemContainer(newItem, connection, topCenterConnector, bottomCenterConnector);
                    AddItemContainer(newItem, topCenterConnector, bottomCenterConnector);
                }

                // Second pass, associate DiagramItem with its parent and
                // add Position it.
                foreach (DiagramItem item in ItemsContainer.Children)
                {
                    DiagramItem di = item;
                    var parentItem = (from DiagramItem c in ItemsContainer.Children where c.Id == di.ParentId select c).FirstOrDefault();
                    if (parentItem != null)
                    {
                        var sinkCtor = (from c in parentItem.Connectors where c.Orientation == ConnectorOrientation.Bottom select c).FirstOrDefault();
                        var sourceCtor = (from c in di.Connectors where c.Orientation == ConnectorOrientation.Top select c).FirstOrDefault();

                        var connection = new Connection()
                        {
                            Source = sourceCtor,
                            Sink = sinkCtor,
                        };
                        connection.Path.Style = ConnectionStyle;
                        di.Connections.Add(connection);
                        parentItem.Connections.Add(connection);
                        ConnectionsContainer.Children.Add(connection.Path2);

                        //// Need to link the Target property of the Connection.
                        //var cnt = di.Connections.FirstOrDefault();
                        //if (cnt != null)
                        //{
                        //    cnt.Sink = parentItem.Connectors.Where(c => c.Orientation == ConnectorOrientation.Bottom).FirstOrDefault();
                        //}

                        //parentItem.ChildItems.Add(di);
                        //di.ParentItem = parentItem;
                    }

                    // Positioning
                    Dispatcher.BeginInvoke(() =>
                    {
                        var location = new Point(di.Left, di.Top);
                        di.MoveItemBy(location, true);
                    });
                }
            }
        }

        #endregion

        internal void AddItemContainer(DiagramItem container, Connection connection,
            Connector topCenterConnector, Connector bottomCenterConnector)
		{
			ItemsContainer.Children.Add(container);
            ConnectorsContainer.Children.Add(topCenterConnector);
            ConnectorsContainer.Children.Add(bottomCenterConnector);
            ConnectionsContainer.Children.Add(connection.Path2);
        }
        internal void AddItemContainer(DiagramItem container, 
            Connector topCenterConnector, Connector bottomCenterConnector)
        {
            ItemsContainer.Children.Add(container);
            ConnectorsContainer.Children.Add(topCenterConnector);
            ConnectorsContainer.Children.Add(bottomCenterConnector);
        }

		public void RemoveItemContainer(DiagramItem container, Line connection)
		{
			ItemsContainer.Children.Remove(container);
            ConnectionsContainer.Children.Remove(connection);
        }

        public void SetSelection(DiagramItem item)
		{
			if (_selectedContainer == item) return; // clicked on already selected instance

            ClearSelection();

			_selectedContainer = item;
			_selectedContainer.IsSelected = true;
			SelectedItem = item.Content;
		}

        private void ClearSelection()
        {
            foreach (DiagramItem item in ItemsContainer.Children)
            {
                item.IsSelected = false;
            }
            _selectedContainer = null;
            SelectedItem = null;
        }

        public int GetMaxZIndex
        {
            get 
            {
                int result = 0;
                if (ItemsContainer.Children.Count > 0)
                {
                    result = (from c in ItemsContainer.Children orderby Canvas.GetZIndex(c) descending select Canvas.GetZIndex(c)).FirstOrDefault(); 
                }
                return result;
            }
        }

    };

    public interface IPosition
    {
        int Id { get; set; }
        int ParentId { get; set; }
        double Top { get; set; }
        double Left { get; set; }
    };

}
